/*******************************************************************************
*                  Copyright (c) 1991-1995 Gemplus Development
*
* Name        : GemDef.h
*
* Description : General definition for GEMPLUS programs
*
* Release     : 4.01.002
*
* Last Modif  : 15/06/95: V4.01.002
* Date        : 06/07/94: V4.00.000
*
********************************************************************************
*
* Warning     :
*
* Remark      :
*
*******************************************************************************/


/*------------------------------------------------------------------------------
Name definition:
   _GEMDEF_H is used to avoid multiple inclusion.
------------------------------------------------------------------------------*/
#ifndef _GEMDEF_H
#define _GEMDEF_H

/*------------------------------------------------------------------------------
Compatibility section:
   G_WINDOWS must be defined for Windows target.
      This can be made
         - by the addition of #include <windows.h> before this file inclusion or
         - by added /DG_WINDOWS as a compilation flag.
------------------------------------------------------------------------------*/
#ifndef G_WINDOWS
   #ifdef _INC_WINDOWS
      #define G_WINDOWS
   #endif

   #ifdef __WINDOWS_H
      #define G_WINDOWS
   #endif
#endif
/*------------------------------------------------------------------------------
   G_DECL is used for function declaration to be adapted to target OS. Today,
      only Windows 3.1 and DOS are supported by this file.
------------------------------------------------------------------------------*/
#ifdef G_WINDOWS
   #define G_DECL FAR PASCAL
#else
   #define G_DECL
#endif
/*------------------------------------------------------------------------------
   G_FAR and G_HUGE are defined to prepare development under 32 bits 
      environment. They are adapted for Borland and Microsoft compilers.
------------------------------------------------------------------------------*/
#ifdef __TURBOC__
   #if __TURBOC__ < 0x410
      #define __far  far
      #define _far   far
      #define __huge huge
      #define _huge  huge
   #endif
#else
   #ifdef _MSC_VER
      #if _MSC_VER < 700
         #define __far  _far
         #define __huge _huge
      #endif
   #else
      #define __far  far
      #define _far   far
      #define __huge huge
      #define _huge  huge
   #endif
#endif
#define G_FAR  __far
#define G_HUGE __huge

/*------------------------------------------------------------------------------
Type definitions:
------------------------------------------------------------------------------*/
#ifndef G_WINDOWS
   typedef unsigned char   BYTE;
   typedef unsigned short  WORD;
   typedef unsigned long   DWORD;
   typedef unsigned short  HINSTANCE;
#endif

typedef unsigned char      WORD8;
typedef unsigned short     WORD16;
typedef unsigned long      WORD32;

typedef signed   char      INT8;
typedef signed   short     INT16;
typedef signed   short     INTEGER;
typedef signed   long      INT32;

#ifndef G_WINDOWS
   typedef signed short    BOOL;
#endif
#ifndef FALSE
   #define FALSE           0
#endif
#ifndef TRUE
   #define TRUE            1
#endif

#ifdef FILE
   typedef  FILE G_FAR    *TEXT;
#else                     
   #ifdef _FILE_DEFINED
      typedef  FILE G_FAR *TEXT;
   #endif
#endif

typedef signed char G_FAR *STRING;

/*------------------------------------------------------------------------------
   TLV_TYPE structure holds:
      - a type field   (T) which indicates the V field type:
      - a length field (L) which indicates the size of the V field, in  bytes.
      - a value field  (V) which is the data part of the message.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD8        T;
   WORD8        L;
   void  G_FAR *V;

} TLV_TYPE;


/*------------------------------------------------------------------------------
Constant definitions:
   EOS is the End Of String character.
   MAX_FILE_NAME is the maximal length for a full file name, with path.
   MAX_LINE_LENGTH is the maximal length for an ASCII line in our configuration 
      files.
------------------------------------------------------------------------------*/
#define EOS                     '\0'
#define MAX_FILE_NAME           129
#define MAX_LINE_LENGTH         120

#define OFF                       0
#define ON                        1
#define CLEAR                     2

#ifndef NULL
   #define NULL                   0
#endif


/*------------------------------------------------------------------------------
Connexion section:
   COM_TYPE defines the today supported Gemplus peripheral connection type.
------------------------------------------------------------------------------*/
typedef enum
{
   G_SERIAL,
   G_LPT,
   G_SCSI,
   G_PCCARD,
   G_ASPI

} COM_TYPE;
/*------------------------------------------------------------------------------
   Serial port definitions.
      COM_SERIAL structure gathers the serial port parameters:
         - Port holds the port key word or directly the port address.
           Under Windows, it is not possible to give directly the address.
         - BaudRate holds the baud rate for the selected communication port.
         - ITNumber is used to select the interrupt which will be used to drive
           the selected port.
           Under Windows, it is not possible to give directly the ITNumber.
      G_COMx must be used for automatic address selection.
      GCRx_y must be used with Gemplus GRI200 card. The first digit declares 
         serial port in use and the second digit declares card port in use.
      DEFAULT_IT is used for automatic interrupt selection.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD16 Port;
   WORD32 BaudRate;
   WORD16 ITNumber;
   
} COM_SERIAL;

#define G_COM1        1
#define G_COM2        2
#define G_COM3        3
#define G_COM4        4
#define GRI1_1     0x11
#define GRI1_2     0x21
#define GRI2_1     0x12
#define GRI2_2     0x22
#define GRI3_1     0x13
#define GRI3_2     0x23
#define GRI4_1     0x14
#define GRI4_2     0x24

#define DEFAULT_IT 0xFF
/*------------------------------------------------------------------------------
   LPT port definitions.
      COM_LPT structure gathers the LPT port parameters:
         - Port holds the port key word or directly the port address.
      G_LPTx must be used for automatic address selection.
------------------------------------------------------------------------------*/
typedef struct
{
   INT16 Port;
   
} COM_LPT;

#define G_LPT1 -1
#define G_LPT2 -2
#define G_LPT3 -3
#define G_LPT4 -4
/*------------------------------------------------------------------------------
   SCSI port definitions.
      COM_SCSI structure gathers the SCSI port parameters:
         - Address holds the SCSI port base address.
         - HostId holds the host SCSI identifier and PeriphId holds the target
           SCSI identifier in {0,1,2,3,4,5,6,7}
------------------------------------------------------------------------------*/
typedef struct
{
   WORD16 Address;
   WORD16 HostId;
   WORD16 PeriphId;
   
} COM_SCSI;
/*------------------------------------------------------------------------------
   PCCARD port definitions.
      COM_PCCARD structure gathers the PCMCIA port parameters:
         - Socket holds a logical socket number from 0 to 5.
         - ITNumber is used to select the interrupt which will be used to drive
           the selected port.
      DEFAULT_IT is used for automatic interrupt selection.
      NO_IT is used when the PCCARD is not interrupt driven.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD16 Socket;
   WORD16 ITNumber;
   
} COM_PCCARD;

#define NO_IT 0xFE
/*------------------------------------------------------------------------------
   ASPI SCSI  port definitions.
      COM_ASPI structure gathers the SCSI port parameters (ASPI Driven):
         - HostId holds the host SCSI identifier and PeriphId holds the target
           SCSI identifier in {0,1,2,3,4,5,6,7}
         - BoardNumber holds the SCSI Board Number if more than one is installed.
           The first board is associated to 0.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD16 HostId;
   WORD16 PeriphId;
   WORD16 BoardNumber;
   
} COM_ASPI;


/*------------------------------------------------------------------------------
Card section:
------------------------------------------------------------------------------*/
#define  ISOCARD               0xF0
#define  COSCARD               0xF0

#define  GPM256                0xF1
#define  GPM416                0xF2    /* 0V on the pad fuse C4 of IFD.       */
#define  GPM896                0xF3    /* Simulate a fuse blown on ICC.       */

#define  GSM1K                 0xF4
#define  GSM4K                 0xF6

#define  GPM103                0xF5

#define  GPM416R               0xF8    /* 5V on the pad fuse C4 of IFD.       */
#define  GPM896R               0xF9    /* To have the real comportment.       */

#define  GFM                   0xFA

#define  GAM                   0xFB


/*------------------------------------------------------------------------------
Reader section:
   SECURITY macro gives access to reader embedded security modules. It must be
      use as SECURITY(GCR500) for example.
   PERIPHERAL macro gives access to reader embedded peripherals, like security
      module.
      SECURITY(GCR500) = PERIPHERAL(1,GCR500).
------------------------------------------------------------------------------*/
#define GCR200                 0x03

#define GCR400FD_A             0x03
#define GCR400FD_B             0x05

#define GCR400                 0x04
#define GCR500                 0x05
#define GCR610                 0x06
#define GCR680                 0x07

#define GPR                    0x0C
#define GPR400                 0x0D

#define IFD140_200             0xFF
#define IFD140_400             0xFD

#define IFD220                 0xFE

#define TLP224                 0x02

#define JERRY                  0x0B

#define TRACE_SYR              0x80

#define SECURITY(x)            (0x0100 | (x))
#define PERIPHERAL(x,y)        ((0x0080 << (x)) | (y))

                                         
/*------------------------------------------------------------------------------
Product Interface Library section:
   MAX_RESET_LEN holds the maximal length for an answer to reset.
   MAX_APDU_LEN is the maximal length for an APDU buffer. The maximal value, 
      according to ISO standard is 65544 but today, low level drivers are 
      limited by the size of a segment.
   MAX_IFD_STRING holds the maximal length for an IFD identier.
   MAX_OFD_SPECIFIC holds the maximal size for the IFD specific field.
   MAX_DUMMY_CHANNEL and MAX_DUMMY_SESSION are used in RFU fields.
------------------------------------------------------------------------------*/
#define MAX_RESET_LEN        33
#define MAX_APDU_LEN      65535lu
#define MAX_IFD_STRING      100
#define MAX_IFD_SPECIFIC    100
#define MAX_DUMMY_CHANNEL   100
#define MAX_DUMMY_SESSION   100 - MAX_RESET_LEN
/*------------------------------------------------------------------------------
   COMMAND_LEN define the size of an ICC command.
   NULL_LEN is a value used with Gemplus specific ICC products.
------------------------------------------------------------------------------*/
#define COMMAND_LEN          4
#define NULL_LEN    0xFFFFFFFFlu
/*------------------------------------------------------------------------------
   Defines the available protocols between host and reader:
      TLP224_PROTOCOL or BLOCK_PROTOCOL on GCI400.
      PCCARD_PROTOCOL on GPR or GPR400
------------------------------------------------------------------------------*/
#define TLP224_PROTOCOL      0
#define BLOCK_PROTOCOL       1
#define PCCARD_PROTOCOL      2
/*------------------------------------------------------------------------------
   G4_CHANNEL_PARAM structure holds:
      - IFDType holds the reader connected: See reader section.
      - IFDBaudRate holds the maximal baud rate to use with the selected reader. 
        If the reader cannot change dynamically its communication baud rate, 
        this value is used as the selected baud rate.
      - IFDMode selects the connection mode for the reader.
      - Comm holds the communication parameters. A Dummy field is added for a 
        future use.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD16   IFDType;
   WORD32   IFDBaudRate;
   COM_TYPE IFDMode;
   union
   {
      COM_SERIAL Serial;
      COM_LPT    Lpt;
      COM_SCSI   Scsi;
      COM_PCCARD PcCard;
      COM_ASPI   Aspi;
      WORD8      Dummy[MAX_DUMMY_CHANNEL];
   
   }        Comm;

} G4_CHANNEL_PARAM;          
/*------------------------------------------------------------------------------
   G4_SESSION_PARAM structure holds:
      - CardType holds the awaited card type: See card section.
      - ApduLenMax holds the maximal APDU length supported according to ICC.
      - ResetLen holds the Answer To Reset length.
      - HistLen holds the number of available historical characters.
      - HistOffset holds offset for the first historical byte in ATR.
      - ResetChar holds the Answer To Reset characters.
      - Dummy field is added for a future use.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD16 ICCType;
   WORD32 ApduLenMax;
   WORD16 ResetLen;
   WORD16 HistLen;
   WORD16 HistOffset;
   WORD8  ResetChar[MAX_RESET_LEN];
   WORD8  Dummy[MAX_DUMMY_SESSION];
   
} G4_SESSION_PARAM;
/*------------------------------------------------------------------------------
   G4_APDU_COMM structure holds:
      - Command holds the command bytes to send to ICC.
      - LengthIn holds the number of bytes to send to ICC. The allowed range is
        { 0 .. 65535 }. The NULL_LEN is used for Gemplus specific products.
      - DataIn holds the bytes to send to ICC.
        WARNING: the user must allocate this buffer.
      - LengthExpected memorises the maximal length expected in the ICC 
        response. The allowed range is { 0 .. 65536}.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD8         Command[COMMAND_LEN];
   WORD32        LengthIn;
   WORD8  G_FAR *DataIn;
   WORD32        LengthExpected;

} G4_APDU_COMM;
/*------------------------------------------------------------------------------
G4_APDU_RESP structure holds:
   - LengthOut is the real number of received bytes.
   - DataOut holds the received bytes. 
     WARNING: the user must allocate this buffer.
   - Status holds the two status bytes SW1 and SW2.
------------------------------------------------------------------------------*/
typedef struct
{
   WORD32        LengthOut;
   WORD8  G_FAR *DataOut;
   WORD16        Status;

} G4_APDU_RESP;
/*------------------------------------------------------------------------------
G4_IFD_STATUS structure holds:
   - Parameters holds the communication parameters given to G4_OpenChannel.
   - Protocol return the currently in use protocol with the reader.
     Available protocols are TLP224_PROTOCOL, BLOCK_PROTOCOL;
   - ExchangeSize holds the communication buffer size currently available in 
     the reader.
   - OSLength holds the number of bytes available in OSString.
   - OSString[MAX_IFD_STRING] holds a null terminated string which identifies
     the OS name and version.
   - SpecificLength holds the number of updated byte in Dummy field.
   - Specific[MAX_IFD_SPECIFIC] holds proprietary information for the selected 
     IFD.
------------------------------------------------------------------------------*/
typedef struct
{
   G4_CHANNEL_PARAM Parameters;
   WORD16           Protocol;
   WORD32           ExchangeSize;
   WORD16           OSLength;
   WORD8            OSString[MAX_IFD_STRING];
   WORD16           SpecificLength;
   WORD8            Specific[MAX_IFD_SPECIFIC];

} G4_IFD_STATUS;

#endif
